unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Chart, TeEngine, Series, TeeGDIPlus;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    BIso: TButton;
    BEdit: TButton;
    BResize: TButton;
    BResetChartRect: TButton;
    procedure FormCreate(Sender: TObject);
    procedure BIsoClick(Sender: TObject);
    procedure BEditClick(Sender: TObject);
    procedure BResizeClick(Sender: TObject);
    procedure BResetChartRectClick(Sender: TObject);
  private
    { Private declarations }
    procedure MakeIsoAxis(Chart: TCustomChart); overload;
    procedure MakeIsoAxis(Series: TChartSeries); overload;
    procedure MakeIsoAxis(Vertical, Horizontal: TChartAxis); overload;
    procedure MakeIsoChartRect;
    procedure Chart1Zoom(Sender: TObject);
    procedure Chart1UndoZoom(Sender: TObject);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses EditChar, Math;

var Chart1: TChart;
    Series1: TPointSeries;

procedure TForm1.FormCreate(Sender: TObject);
var t: Integer;
begin
  Chart1:=TChart.Create(Self);
  Chart1.Parent:=Self;
  Chart1.Align:=alClient;

  Chart1.View3D:=false;
  Chart1.Legend.Visible:=false;

  Series1:=Chart1.AddSeries(TPointSeries) as TPointSeries;

  with Chart1.Axes.Left do
  begin
    Grid.Style:=psSolid;
    Grid.Color:=clBlack;
    Increment:=0.1;
    LabelsSeparation:=0;
  end;

  with Chart1.Axes.Bottom do
  begin
    Grid.Style:=psSolid;
    Grid.Color:=clBlack;
    Increment:=0.1;
    LabelsSeparation:=0;
  end;

  Series1.Clear;
  for t:=0 to 50-1 do Series1.AddXY(t/50, Random*0.5);

  Chart1.OnZoom:=Chart1Zoom;
  Chart1.OnUndoZoom:=Chart1UndoZoom;
end;

procedure TForm1.BIsoClick(Sender: TObject);
begin
  MakeIsoAxis(Series1);
end;

procedure TForm1.Chart1Zoom(Sender: TObject);
begin
  inherited;
  MakeIsoAxis(Series1);
end;

procedure TForm1.Chart1UndoZoom(Sender: TObject);
begin
  inherited;
  MakeIsoAxis(Series1);
end;

procedure TForm1.MakeIsoAxis(Vertical, Horizontal: TChartAxis);
var tmpX,
    tmpY,
    XRange,
    YRange,
    Offset,
    XYScreen : Double;
begin
  with Vertical.ParentChart do
  if (ChartHeight>0) and (ChartWidth>0) then
  begin
    With Horizontal do XRange:=Maximum-Minimum;

    tmpX:=(XRange/ChartWidth);

    {$IFDEF CLX}
    XYScreen:=1024.0/768.0;  //TODO
    {$ELSE}
    XYScreen:=1.0*(GetDeviceCaps(Canvas.Handle,HORZSIZE)/Screen.Width)/
                  (GetDeviceCaps(Canvas.Handle,VERTSIZE)/Screen.Height);
    {$ENDIF}

    With Vertical do YRange:=Maximum-Minimum;

    tmpY:=(YRange/ChartHeight)*XYScreen;

    if tmpX>tmpY then
    begin
      if tmpY<>0 then
      begin
        Offset:=((YRange*tmpX/tmpY)-YRange)/2.0;
        With Vertical do SetMinMax(Minimum-Offset,Maximum+Offset);
      end;
    end
    else
    if tmpX<tmpY then
    begin
      if tmpX<>0 then
      begin
        Offset:=((XRange*tmpY/tmpX)-XRange)/2.0;
        With Horizontal do SetMinMax(Minimum-Offset,Maximum+Offset);
      end;
    end;
  end;
end;

procedure TForm1.MakeIsoAxis(Series: TChartSeries);
begin
  MakeIsoAxis(Series.GetVertAxis, Series.GetHorizAxis);
end;

procedure TForm1.MakeIsoAxis(Chart: TCustomChart);
begin
  MakeIsoAxis(Chart.Axes.Left, Chart.Axes.Bottom);
end;

procedure TForm1.MakeIsoChartRect;
var tmpW, tmpH, i: Integer;
begin
  MakeIsoAxis(Series1);
  Chart1.Draw;

  tmpW:=0;
  tmpH:=0;
  for i:=0 to Chart1.SeriesCount-1 do
  begin
    tmpW:=Max(tmpW, Abs(Chart1[i].GetHorizAxis.CalcPosValue(Chart1[i].XValues.MaxValue) - Chart1[i].GetHorizAxis.CalcPosValue(Chart1[i].XValues.MinValue)));
    tmpH:=Max(tmpH, Abs(Chart1[i].GetVertAxis.CalcPosValue(Chart1[i].YValues.MaxValue) - Chart1[i].GetVertAxis.CalcPosValue(Chart1[i].YValues.MinValue)));
  end;

  Chart1.CustomChartRect:=True;
  Chart1.ChartRect.Left:=20;
  Chart1.ChartRect.Top:=20;
  Chart1.ChartRect.Right:=Chart1.ChartRect.Left+tmpW;
  Chart1.ChartRect.Bottom:=Chart1.ChartRect.Top+tmpH;

  for i:=0 to Chart1.SeriesCount-1 do
  begin
    Chart1[i].GetHorizAxis.Automatic:=True;
    Chart1[i].GetVertAxis.Automatic:=True;
  end;
end;

procedure TForm1.BResetChartRectClick(Sender: TObject);
begin
  Chart1.CustomChartRect:=false;
  Chart1.Repaint;
end;

procedure TForm1.BResizeClick(Sender: TObject);
begin
  MakeIsoChartRect;
end;

procedure TForm1.BEditClick(Sender: TObject);
begin
  EditChart(Self, Chart1);
end;

end.
